from .test_matriculation import MatriculationTestCase
from .test_configuration import ConfigurationTestCase
from .test_party import PartyTestCase
from .test_report import ReportTestCase

__all__ = ['MatriculationTestCase', 'ReportTestCase', 'ConfigurationTestCase', 'PartyTestCase']
